//-----------------------------------------------------------------------------
// Copyright  1999 - Philip Howard
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	rmd160
// version	1.2.0
// homepage	http://phil.ipal.org/freeware/rmd160/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	phil@ipal.org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------

#include "rmd160_lib.h"

//-----------------------------------------------------------------------------
// function	rmd160_copy
//
// purpose	Copy the state from one RMD160 object to another.
//
// arguments	1 (RMD160) reference handle to target RMD160 instance.
//		2 (RMD160) reference handle to source RMD160 instance.
//
// returns	(RMD160) target RMD160 object, allocated if needed.
//-----------------------------------------------------------------------------
// function	rmd160_dup
//
// purpose	Duplicate the state from one RMD160 object to a new one.
//
// arguments	1 (RMD160) reference handle to source RMD160 instance.
//
// returns	(RMD160) newly created RMD160 object.
//-----------------------------------------------------------------------------
// function	rmd160_init
//
// purpose	Initialize the state of a specified RMD160 object.
//
// arguments	1 (RMD160) reference handle to target RMD160 instance.
//
// returns	(RMD160) target RMD160 object, allocated if needed.
//-----------------------------------------------------------------------------
// function	rmd160_new
//
// purpose	Create a new, initialized, RMD160 object.
//
// arguments	-none-
//
// returns	(RMD160) newly created RMD160 object.
//-----------------------------------------------------------------------------

#define rmd160_dup(s) rmd160_copy(NULL,(s))
#define rmd160_init(t) rmd160_copy((t),NULL)
#define rmd160_new() rmd160_copy(NULL,NULL)

#define FUNCTION_NAME "rmd160_copy"

RMD160
rmd160_copy (
	RMD160		target_p
,
	RMD160		source_p
)
{
    if ( ! target_p ) {
	if ( ! ( target_p = malloc( sizeof (struct rmd160_object) ) ) ) {
	    return NULL;
	}
    }

    if ( source_p ) {
	target_p->state[ 0 ] = source_p->state[ 0 ];
	target_p->state[ 1 ] = source_p->state[ 1 ];
	target_p->state[ 2 ] = source_p->state[ 2 ];
	target_p->state[ 3 ] = source_p->state[ 3 ];
	target_p->state[ 4 ] = source_p->state[ 4 ];
	{
	    int i;
	    for ( i = 0 ; i < 16 ; ++ i ) {
		target_p->data[ i ] = source_p->data[ i ];
	    }
	}
	target_p->len[ 0 ] = source_p->len[ 0 ];
	target_p->len[ 1 ] = source_p->len[ 1 ];
	target_p->bpos = source_p->bpos;
	target_p->wptr = source_p->wptr - source_p->data + target_p->data;
	target_p->wend = 16 + target_p->data;
    }
    else {
	target_p->state[ 0 ] = RMD160_INIT0;
	target_p->state[ 1 ] = RMD160_INIT1;
	target_p->state[ 2 ] = RMD160_INIT2;
	target_p->state[ 3 ] = RMD160_INIT3;
	target_p->state[ 4 ] = RMD160_INIT4;
	{
	    int i;
	    for ( i = 0 ; i < 16 ; ++ i ) {
		target_p->data[ i ] = 0U;
	    }
	}
	target_p->len[ 0 ] = 0U;
	target_p->len[ 1 ] = 0U;
	target_p->bpos = 0;
	target_p->wptr = target_p->data;
	target_p->wend = 16 + target_p->data;
    }

    return target_p;
}

#undef FUNCTION_NAME
