//-----------------------------------------------------------------------------
// Copyright  1999 - Philip Howard
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	rmd160
// version	1.2.0
// homepage	http://phil.ipal.org/freeware/rmd160/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	phil@ipal.org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------

#include "rmd160_lib.h"

//-----------------------------------------------------------------------------
// function	rmd160_sum_base16
//
// purpose	Get the resultant checksum of the RMD160 object as
//		an array of 40 4-bit character encoded values.
//
// arguments	(RMD160) reference handle to RMD160 instance.
//		(char *) pointer to 40 base16 characters where
//				the checksum result is to be stored.
//
// returns	(char *) pointer to returned checksum result.
//		(char *) NULL if error
//-----------------------------------------------------------------------------

#define FUNCTION_NAME "rmd160_sum_base16"

char *
rmd160_sum_base16 (
	RMD160		source_p
,
	char *		result_p
)
{
    static char		encode	[16]	= "0123456789abcdef";

    rmd160uint32	temp	[5]	;

    rmd160uint32 *	ptemp		;



    //-- Make sure we have a result buffer.
    if ( ! result_p && ! ( result_p = malloc( 41 ) ) ) return NULL;


    //-- Get the results as words.
    if ( ! rmd160_sum_words( source_p , temp ) ) return NULL;

    //-- Convert words to characters in the result buffer.
    ptemp = temp;
    {
	int i;
	for ( i = 0 ; i < 5 ; ++ i ) {
	    register rmd160uint32 w;

	    //------------------------------------------------------
	    // The standard cetal/hexadecimal result is a mixture of
            // digits in big-endian order and bytes in little-endian
            // order.  It makes things just slightly confusing since
	    // the RIPEMD-160 documentation just says little-endian.
	    // Note the storage order: 1,0,3,2,5,4,7,6.
	    //------------------------------------------------------
	    w = * ptemp ++;
	    result_p[1] = encode[ 0xf & w ];
	    w >>= 4;
	    result_p[0] = encode[ 0xf & w ];
	    w >>= 4;
	    result_p[3] = encode[ 0xf & w ];
	    w >>= 4;
	    result_p[2] = encode[ 0xf & w ];
	    w >>= 4;
	    result_p[5] = encode[ 0xf & w ];
	    w >>= 4;
	    result_p[4] = encode[ 0xf & w ];
	    w >>= 4;
	    result_p[7] = encode[ 0xf & w ];
	    w >>= 4;
	    result_p[6] = encode[ 0xf & w ];
	    result_p += 8;
	}
    }
    * ( result_p ++ ) = '\0';

    //-- All done, return.
    return result_p;

}

#undef FUNCTION_NAME
